<?php
class Zend_Asn1_Base {

    /**
     * Constants
     */
    const CONTEXT_SPECIFIC  = "\x80";
    const CONSTRUCTED       = "\x20";
    const MASK              = "\x1F";
    
     /**
     * @var byte
     */
    protected $_identifier = "\x00";
    
    /**
     * @var string|bytes
     */
    protected $_value = '';
    
    /**
     * @var array Array of Asn1_Base
     */
    public $nodes = array();

     /**
     * Constructor
     *
     * @param string|byte $symbol The identifier byte
     * @param string $value The value in binary form
     * @param array $nodes Array of Asn1_Base instances
     */
    public function __construct($symbol, $value='', $nodes = array()) {
        $this->setSymbol($symbol);
        $this->setValue($value);
        $this->setNodes($nodes);
    }


    /**
     * Set the identifier byte
     *
     * @param string|byte $symbol
     */
    public function setSymbol($symbol) {
        $this->_identifier = $symbol;
    }

    /**
     * Get the identifier byte
     *
     * @return string|byte
     */
    public function getSymbol() {
        return $this->_identifier;
    }

    /**
     * Set the value
     *
     * @param string $value Value in binary form
     */
    public function setValue($value) {
        $this->_value = $value;
    }

    /**
     * Get the value
     *
     * @return string
     */
    public function getValue() {
        return $this->_value;
    }

    /**
     * Parses a BER encoded string
     *
     * @param string $s
     * @return Asn1_Base|array
     */
    static public function parse($s) {
        $res = array();
        
        $cur = 0;
        $sLenght = strlen($s);
        
        while ($cur < $sLenght) {
            $symbol = $s[$cur++];
            $subType = $symbol & self::MASK;
            
            $len = ord($s[$cur++]);
            if (($len & 128) == 128) {
    			$tempLen = 0;
    			for ($x=0; $x < ($len & (128-1)); $x++){
    				$tempLen = ord($s[$cur++]) + ($tempLen * 256);
    			}
    			$len = $tempLen;
    		}
    		$value = (string)substr($s, $cur, $len);
    		
    		switch ($subType) {
    		    case (($symbol & self::CONSTRUCTED) != "\x00"):
    		    case Zend_Asn1_Sequence::IDENTIFIER:
                case Zend_Asn1_Set::IDENTIFIER:
                    $res[] = new self($symbol, '', self::parse($value));
                    break;
    		    default:
    		        $res[] = new self($symbol, $value);
    		}
    		
    		$cur = $cur + $len;
        }
		
        if (count($res) == 1)
            return current($res);
        return $res;
    }
    
    /**
     * Decodes an OID
     *
     * @param string $oid OID in binary form
     * @return string The OID in dot form
     */
    static public function decode($oid) {
        $strOid = '';
        $cur = 0;
        $len = strlen($oid);
        $b = ord($oid[$cur++]);
        
        $strOid .= (string)((int)($b/40));
        $strOid .= '.' . (string)((int)($b%40));
        
        while ($cur < $len) {
            $v = self::_decodeValue($oid, $cur);
            $strOid .= '.' . $v;
        }
        
        return $strOid;
    }
    
    static protected function _decodeValue($oid, &$cur) {
        $v = 0;
        for (;;) {
            $b = ord($oid[$cur++]);
            $v <<= 7;
            $v += ($b & 0x7f);
            if (($b & 0x80) == 0)
                return $v;
        }
    }
    
    /**
     * Encodes an OID
     *
     * @param string $strOid OID in dot form
     * @return string OID in binary form
     */
    static public function encode($strOid) {
        $parts = explode(".", $strOid);
		$oid = chr(40 * $parts[0] + $parts[1]);
		for ($i = 2, $len = count($parts); $i < $len; $i++) {
			$current = (int)$parts[$i];
			if ($current-1 > 0x80){
				$add = chr($current % 0x80);
				$current = floor($current / 0x80);
				while ($current > 127){
					$add = chr(($current % 0x80) | 0x80).$add;
					$current = floor($current / 0x80);
				}
				$add = chr(($current % 0x80) | 0x80).$add;
				$oid .= $add;
			} else {
				$oid .= chr($current);
			}
		}
		return $oid;
    }
           
    /**
     * Returns the length byte in DER encoding
     *
     * @param integer $len
     * @return string
     */
    static public function lengthToDer($len) {
        if ($len < 128) {
            return chr($len);
        } else {
            $out = '';
			while ($len >= 256) {
			    $out = chr($len % 256) . $out;
				$len = floor($len / 256);
			}
			$out = chr($len).$out;
			$out = chr(128 | strlen($out)).$out;
			return $out;
        }
    }    
   
    /**
     * Set nodes
     *
     * @param array $nodes Array of Asn1_Base instances
     */
    public function setNodes($nodes = array()) {
        if (!is_array($nodes))
            $nodes = array($nodes);
            
        $this->nodes = array();
        foreach ($nodes AS $node) {
            $this->addNode($node);
        }
    }
       
    /**
     * Add a node
     *
     * @param Asn1_Base $node
     */
    public function addNode(Zend_Asn1_Base $node) {
        $this->nodes[] = $node;
       // $node->setParent($this);
    }
    
    
    /**
     * Returns the BER encoded string
     *
     * @return string
     */
    public function __toString() {
        $value = ''; 
        $subType = $this->_identifier & self::MASK;
        
        switch ($subType) {
            case (($this->_identifier & self::CONSTRUCTED) != "\x00"):
		    case Zend_Asn1_Sequence::IDENTIFIER:
            case Zend_Asn1_Set::IDENTIFIER:
                if (count($this->nodes) > 0) {
                    foreach ($this->nodes as $node) {
                    	$value .= (string)$node->__toString();
                    }
                } else {
                    $value .= (string)$this->_value;
                }
                break;
            default:
                $value = $this->_value;
        }
        
        $len = self::lengthToDer(strlen($value));
        
        return $this->_identifier.$len.$value;
    }
}
